"""A sfg with delays and interesting layout for precedence list generation.
     .                                          .
IN1>--->C0>--->ADD1>----------+--->A0>--->ADD4>--->OUT1
     .           ^            |            ^    .
     .           |            T1           |    .
     .           |            |            |    .
     .         ADD2<---<B1<---+--->A1>--->ADD3  .
     .           ^            |            ^    .
     .           |            T2           |    .
     .           |            |            |    .
     .           +-----<B2<---+--->A2>-----+    .
"""

from b_asic.core_operations import Addition, ConstantMultiplication
from b_asic.special_operations import Delay, Input, Output
from b_asic.signal_flow_graph import SFG
from b_asic.schedule import Schedule


in1 = Input("IN1")
c0 = ConstantMultiplication(5, in1, "C0")
add1 = Addition(c0, None, "ADD1")
# Not sure what operation "Q" is supposed to be in the example
T1 = Delay(add1, 0, "T1")
T2 = Delay(T1, 0, "T2")
b2 = ConstantMultiplication(0.2, T2, "B2")
b1 = ConstantMultiplication(0.3, T1, "B1")
add2 = Addition(b1, b2, "ADD2")
add1.input(1).connect(add2)
a1 = ConstantMultiplication(0.4, T1, "A1")
a2 = ConstantMultiplication(0.6, T2, "A2")
add3 = Addition(a1, a2, "ADD3")
a0 = ConstantMultiplication(0.7, add1, "A0")
add4 = Addition(a0, add3, "ADD4")
out1 = Output(add4, "OUT1")

sfg = SFG(inputs=[in1], outputs=[out1], name="Second-order direct form IIR filter")

# Set latencies and exection times
sfg.set_latency_of_type(ConstantMultiplication.type_name(), 2)
sfg.set_latency_of_type(Addition.type_name(), 1)
sfg.set_execution_time_of_type(ConstantMultiplication.type_name(), 1)
sfg.set_execution_time_of_type(Addition.type_name(), 1)

schedule = Schedule(sfg)
